@@ -22,7 +22,6 @@ import (
 const (
 	systemVersion           = "system"
 	latestVersion           = "latest"
-	uninstallableVersionMsg = "uninstallable version: %s"
 	latestFilterRegex       = "(?i)(^Available versions:|-src|-dev|-latest|-stm|[-\\.]rc|-milestone|-alpha|-beta|[-\\.]pre|-next|(a|b|c)[0-9]+|snapshot|master|main)"
 	numericStartFilterRegex = "^\\s*[0-9]"
 	noLatestVersionErrMsg   = "no latest version found"
@@ -31,11 +30,12 @@ const (
 // UninstallableVersionError is an error returned if someone tries to install the
 // system version.
 type UninstallableVersionError struct {
+	toolName    string
 	versionType string
 }
 
 func (e UninstallableVersionError) Error() string {
-	return fmt.Sprintf(uninstallableVersionMsg, e.versionType)
+	return fmt.Sprintf("uninstallable version %s of %s", e.versionType, e.toolName)
 }
 
 // NoVersionSetError is returned whenever an operation that requires a version
@@ -145,13 +145,13 @@ func InstallOneVersion(conf config.Config, plugin plugins.Plugin, versionStr str
 	}
 
 	if versionStr == systemVersion {
-		return UninstallableVersionError{versionType: systemVersion}
+		return UninstallableVersionError{toolName: plugin.Name, versionType: systemVersion}
 	}
 
 	version := toolversions.Parse(versionStr)
 
 	if version.Type == "path" {
-		return UninstallableVersionError{versionType: "path"}
+		return UninstallableVersionError{toolName: plugin.Name, versionType: "path"}
 	}
 	downloadDir := installs.DownloadPath(conf, plugin, version)
 	installDir := installs.InstallPath(conf, plugin, version)
@@ -160,12 +160,13 @@ func InstallOneVersion(conf config.Config, plugin plugins.Plugin, versionStr str
 		return VersionAlreadyInstalledError{version: version, toolName: plugin.Name}
 	}
 
+	concurrency, _ := conf.Concurrency()
 	env := map[string]string{
 		"ASDF_INSTALL_TYPE":    version.Type,
 		"ASDF_INSTALL_VERSION": version.Value,
 		"ASDF_INSTALL_PATH":    installDir,
 		"ASDF_DOWNLOAD_PATH":   downloadDir,
-		"ASDF_CONCURRENCY":     asdfConcurrency(conf),
+		"ASDF_CONCURRENCY":     concurrency,
 	}
 
 	err = os.MkdirAll(downloadDir, 0o777)
@@ -195,6 +196,9 @@ func InstallOneVersion(conf config.Config, plugin plugins.Plugin, versionStr str
 
 	err = plugin.RunCallback("install", []string{}, env, stdOut, stdErr)
 	if err != nil {
+		if rmErr := os.RemoveAll(installDir); rmErr != nil {
+			fmt.Fprintf(stdErr, "failed to clean up '%s' due to %s\n", installDir, rmErr)
+		}
 		return fmt.Errorf("failed to run install callback: %w", err)
 	}
 
@@ -227,21 +231,6 @@ func InstallOneVersion(conf config.Config, plugin plugins.Plugin, versionStr str
 	return nil
 }
 
-func asdfConcurrency(conf config.Config) string {
-	val, ok := os.LookupEnv("ASDF_CONCURRENCY")
-
-	if !ok {
-		val, err := conf.Concurrency()
-		if err != nil {
-			return "1"
-		}
-
-		return val
-	}
-
-	return val
-}
-
 // Latest invokes the plugin's latest-stable callback if it exists and returns
 // the version it returns. If the callback is missing it invokes the list-all
 // callback and returns the last version matching the query, if a query is
@@ -251,33 +240,31 @@ func Latest(plugin plugins.Plugin, query string) (version string, err error) {
 	var stdErr strings.Builder
 
 	err = plugin.RunCallback("latest-stable", []string{query}, map[string]string{}, &stdOut, &stdErr)
-	if err != nil {
-		if _, ok := err.(plugins.NoCallbackError); !ok {
-			return version, err
-		}
-
-		allVersions, err := AllVersionsFiltered(plugin, query)
-		if err != nil {
-			return version, err
-		}
-
-		versions := filterOutByRegex(allVersions, numericStartFilterRegex, true)
-		versions = filterOutByRegex(versions, latestFilterRegex, false)
-
+	if err == nil {
+		versions := parseVersions(stdOut.String())
 		if len(versions) < 1 {
 			return version, errors.New(noLatestVersionErrMsg)
 		}
-
 		return versions[len(versions)-1], nil
 	}
 
-	// parse stdOut and return version
-	allVersions := parseVersions(stdOut.String())
+	// Fallback to list-all if latest-stable fails
+	if _, ok := err.(plugins.NoCallbackError); !ok {
+		return version, err
+	}
+
+	allVersions, err := AllVersionsFiltered(plugin, query)
+	if err != nil {
+		return version, err
+	}
+
 	versions := filterOutByRegex(allVersions, numericStartFilterRegex, true)
 	versions = filterOutByRegex(versions, latestFilterRegex, false)
+
 	if len(versions) < 1 {
 		return version, errors.New(noLatestVersionErrMsg)
 	}
+
 	return versions[len(versions)-1], nil
 }
 
